@@ -21,7 +21,7 @@ class TrelloWarriorClient:
 
     def upload_taskwarrior_task(self, project, taskwarrior_task, trello_list):
         """
-        Upload all contents of Taskwarrior task to a Trello list creating a new card and storing cardid
+        Upload all contents of Taskwarrior task to a Trello list creating a new card and storing cardid and url
 
         :param project: TrelloWarrior project object
         :param taskwarrior_task: Taskwarrior task object
@@ -37,6 +37,8 @@ class TrelloWarriorClient:
             new_trello_card.assign(self.trello_client.whoami)
         taskwarrior_task['trelloid'] = new_trello_card.id
         taskwarrior_task.save()
+        taskwarrior_task.add_annotation('[Trello URL] {}'.format(new_trello_card.short_url))
+        logger.info('Taskwarrior task with ID {} saved as new card in Trello with ID {}'.format(taskwarrior_task['id'], new_trello_card.id))
 
     def fetch_trello_card(self, project, list_name, trello_card):
         """
@@ -57,6 +59,9 @@ class TrelloWarriorClient:
         new_taskwarrior_task['trelloid'] = trello_card.id
         new_taskwarrior_task['trellolistname'] = list_name
         new_taskwarrior_task.save()
+        new_taskwarrior_task.add_annotation('[Trello URL] {}'.format(trello_card.short_url))
+        if trello_card.description:
+            new_taskwarrior_task.add_annotation('[Trello Description] {}'.format(trello_card.description))
         logger.info('Trello card with ID {} saved as new task in Taskwarrior with ID {}'.format(trello_card.id, new_taskwarrior_task['id']))
         if list_name == project.trello_doing_list:
             new_taskwarrior_task.start()
@@ -177,6 +182,38 @@ class TrelloWarriorClient:
         if taskwarrior_task_modified:
             taskwarrior_task.save()
             logger.info('All changes in Taskwarrior task {} saved'.format(taskwarrior_task['id']))
+        # Task annotations <> Trello url and description
+        # WARNING: The task must be saved before play with annotations https://tasklib.readthedocs.io/en/latest/#working-with-annotations
+        taskwarrior_task_annotation_trello_url = None
+        taskwarrior_task_annotation_trello_description = [None, '']
+        if taskwarrior_task['annotations']:
+            for annotation in taskwarrior_task['annotations']:
+                # Look for Trello url
+                if annotation['description'][0:13].lower() == '[trello url] ':
+                    taskwarrior_task_annotation_trello_url = annotation
+                # Look for Trello description
+                if annotation['description'][0:21].lower() == '[trello description] ':
+                    taskwarrior_task_annotation_trello_description = [annotation, annotation['description'][21:]]
+        if taskwarrior_task_annotation_trello_url is None:
+            # No previous url annotated
+            taskwarrior_task.add_annotation('[Trello URL] {}'.format(trello_card.short_url))
+            logger.info('URL of task {} added'.format(taskwarrior_task['id']))
+        elif taskwarrior_task_annotation_trello_url['description'][13:] != trello_card.short_url:
+            # Cannot update annotations (see https://github.com/robgolding/tasklib/issues/91)
+            # Delete old URL an add the new one
+            taskwarrior_task_annotation_trello_url.remove()
+            taskwarrior_task.add_annotation('[Trello URL] {}'.format(trello_card.short_url))
+            logger.info('URL of task {} synchronized'.format(taskwarrior_task['id']))
+        if taskwarrior_task_annotation_trello_description[1] != trello_card.description:
+            if taskwarrior_task['modified'] > trello_card.date_last_activity:
+                # Taskwarrior data is newer
+                trello_card.set_description(taskwarrior_task_annotation_trello_description[1])
+            else:
+                # Trello data is newer (delete old description and add new one)
+                if taskwarrior_task_annotation_trello_description[0] is not None:
+                    taskwarrior_task_annotation_trello_description.remove()
+                taskwarrior_task.add_annotation('[Trello Description] {}'.format(trello_card.description))
+            logger.info('Description of task {} synchronized'.format(taskwarrior_task['id']))
 
     def sync_project(self, project):
         """
